﻿using Google.Protobuf.WellKnownTypes;
using Grpc.Core;
using HealthMonitor.Common;
using HealthMonitor.Common.helper;
using HealthMonitor.Core.Dal;
using HealthMonitor.Core.Pipeline;
using HealthMonitor.Service.Biz.db;
using HealthMonitor.Service.Cache;
using HealthMonitor.Service.Etcd;
using HealthMonitor.Service.Resolver;
using HealthMonitor.Util.Entities.HealthMonitor;
using HealthMonitor.WebApi.Configs;
using HealthMonitor.WebApi.Model.Request;
using Microsoft.AspNetCore.DataProtection.KeyManagement;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.ViewFeatures;
using Microsoft.EntityFrameworkCore.Metadata;
using Microsoft.EntityFrameworkCore.Metadata.Internal;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using TelpoDataService.Util.Entities.GpsCard;
using TelpoDataService.Util.Clients;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using TelpoDataService.Util;
using TelpoDataService.Util.Models;
using TelpoDataService.Util.QueryObjects;
using HealthMonitor.WebApi.Controllers.Api;
using HealthMonitor.Service.Biz;
using HealthMonitor.Model.Service;

namespace HealthMonitor.WebApi.Controllers.HealthMonitor
{
    [ApiExplorerSettings(GroupName = AppConsts.SWAGGER_DOC_HealthMonitor)]
    [Produces("application/json")]
    [Route("api/HealthMonitor/[controller]/[action]")]
    [ApiController]
    public class HmBloodPressConfigManualCalibrationController : ControllerBase
    {
        private readonly ILogger<HmBloodPressConfigManualCalibrationController> _logger;
        private readonly PersonCacheManager _personCacheMgr;
        private readonly TDengineService _serviceTDengine;
        private readonly HttpHelper _httpHelper = default!;

        private readonly IotWebApiService _serviceIotWebApi;

        private readonly GpsCardAccessorClient<GpsDevice> _deviceApiClient;
        private readonly GpsCardAccessorClient<GpsPerson> _gpsPersonApiClient;

        public HmBloodPressConfigManualCalibrationController
            (
            TDengineService serviceDengine,
            PersonCacheManager personCacheMgr, HttpHelper httpHelper,
            GpsCardAccessorClient<GpsPerson> gpsPersonApiClient,
            GpsCardAccessorClient<GpsDevice> deviceApiClient,
            ILogger<HmBloodPressConfigManualCalibrationController> logger,
            IotWebApiService iotWebApiService
            )
        {


            _serviceTDengine = serviceDengine;
            _logger = logger;

            _httpHelper = httpHelper;
            _gpsPersonApiClient = gpsPersonApiClient;
            _deviceApiClient = deviceApiClient;
            _personCacheMgr = personCacheMgr;
            _serviceIotWebApi = iotWebApiService;
           
        }
        [HttpPost]
        public async Task<ApiResponse<object>> Put([FromBody] BloodPressManualCalibration model, [FromHeader] string requestId)
        {

            #region 设备合法性

#if DEBUG

#else
            var param = new GeneralParam
            {
                Filters = new List<QueryFilterCondition>
                        {
                            new QueryFilterCondition
                            {
                                Key=nameof(GpsDevice.Serialno),
                                Value=model.Imei,
                                ValueType=QueryValueTypeEnum.String,
                                Operator=QueryOperatorEnum.Equal
                            }
                        }
            };

            var device = await _deviceApiClient.GetFirstAsync(param, new RequestHeader { RequestId = requestId }).ConfigureAwait(false);

            if (device == null)
            {

                _logger.LogError($"非法设备:{model.Imei}");
                return ApiResponse<object>.Fail(500, $"非法设备:{model.Imei}");
            }
#endif

#endregion

            #region 重置设备
            if (model.ManualSystolicRefValue.Equals(0) && model.ManualDiastolicRefValue.Equals(0))
            {
                return await IotSetBloodPressCalibrationConfigResponseAsync(model.Imei, 0, 0, 0, 0).ConfigureAwait(false);

                //return ApiResponse<object>.Success(new
                //{
                //    imei = model.Imei,
                //    systolicCalibrationValue = 0,  //收缩压标定值，值为0 表示不生效
                //    diastolicCalibrationValue = 0, //舒张压标定值，值为0表示不生效
                //    systolicIncValue = 0, //收缩压显示增量，值为0 表示不生效
                //    diastolicIncValue = 0   //舒张压显示增量，值为0 表示不生效
                //});
            }

            #endregion

            #region 数据合法性
            if (model.ManualSystolicRefValue.Equals(0) || model.ManualDiastolicRefValue.Equals(0))
            {
                return ApiResponse<object>.Fail(501, $"数据非法,{JsonConvert.SerializeObject(model)}");
            }
            #endregion

            #region 计算增量值
            var imei = model.Imei;
            // 计算增量值
            int systolicRefValue = model.ManualSystolicRefValue;//?
            int diastolicRefValue = model.ManualDiastolicRefValue;//?
                                                                  // 没有血压数据时增量值为0
            int systolicInc;
            int diastolicInc;
            int systolicAvg;
            int diastolicAvg;
            //int duration = 7;
            long duration = 7 * 24 * 3600 * 1000;
            TimeSpan ts = TimeSpan.FromMilliseconds(duration);
            DateTime endTime = DateTime.Now; //测试
            DateTime startTime = endTime - ts;

            var lastPush = await _serviceTDengine.GetLastAsync("stb_hm_bp_push_ref_inc_value", $"serialno='{imei}' order by ts desc");
            
            // A.没有下推记录
            if (lastPush?.Count == 0 || lastPush == null)
            {
                //#region 初始化remarks
                //// 读数据库,remarks为空就写commandValue,更新缓存
                //var resFlag = await _serviceIotWebApi.UpdatePersonRemarksAsync(imei, systolicRefValue, diastolicRefValue).ConfigureAwait(false);
                //if (resFlag)
                //{
                //    _logger.LogInformation($"更新gps_person remarks和缓存");
                //}
                //else
                //{
                //    _logger.LogInformation($"更新gps_person remarks和缓存失败");
                //}
                //#endregion

                int count = await _serviceTDengine.GetCount("stb_hm_bloodpress", $"serialno='{imei}' ");
                // A.1 没有下推记录,没有测量记录,下推增量值 0
                if (count.Equals(0))
                {
                    _logger.LogInformation("A.1 没有下推记录,没有测量记录,下推增量值 0");
                    return await IotSetBloodPressCalibrationConfigResponseAsync(model.Imei, systolicRefValue, diastolicRefValue, 0, 0).ConfigureAwait(false);
                }
                // A.2 没有下推记录,测量记录,正常计算增量值 
                else
                {
                    // 平均值: 计算去除最大值 MAX和最小值 MIN 和异常值 ABN 的平均值
                    systolicAvg = await _serviceTDengine.GetAvgExceptMaxMinValueAsync("systolic_value", "stb_hm_bloodpress", $"ts>='{startTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and ts <='{endTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and serialno='{imei}' and systolic_value < {systolicRefValue} ");
                    diastolicAvg = await _serviceTDengine.GetAvgExceptMaxMinValueAsync("diastolic_value", "stb_hm_bloodpress", $"ts>='{startTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and ts <='{endTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and serialno='{imei}' and diastolic_value < {diastolicRefValue}");

                    // 数据异常,不修改增量值
                    if (systolicAvg.Equals(0) || diastolicAvg.Equals(0))
                    {
                        _logger.LogInformation(" A.2 没有下推记录,有测量记录,正常计算增量值 ,平均值为0, 增量值将按0下发");
                        return await IotSetBloodPressCalibrationConfigResponseAsync(model.Imei, systolicRefValue, diastolicRefValue, 0, 0).ConfigureAwait(false);

                    }
                    // 数据正常
                    else
                    {
                        // 偏移参数
                        var avgOffset = 0.25M;
                        var systolicAvgOffset = avgOffset;
                        var diastolicAvgOffset = avgOffset;


                        // 增量值=（标定值-平均值）* 0.25
                        systolicInc = systolicAvg.Equals(0M) ? 0 : (int)((systolicRefValue - systolicAvg) * systolicAvgOffset)!;
                        diastolicInc = diastolicAvg.Equals(0M) ? 0 : (int)((diastolicRefValue - diastolicAvg) * diastolicAvgOffset)!;
                        _logger.LogInformation(" A.2 没有下推记录,有测量记录,正常计算增量值 ,增量值将正常下发");
                        return await IotSetBloodPressCalibrationConfigResponseAsync(imei, systolicRefValue, diastolicRefValue, systolicInc, diastolicInc).ConfigureAwait(false);
                    }
                }
            }
            // B.有下推记录(更改过个人信息)
            else
            {
                // B.1. 设备已经被重新设置
                if (
                    (int)lastPush![2] == 0
                    && (int)lastPush![3] == 0
                    && (int)lastPush![4] == 0
                    && (int)lastPush![5] == 0
                    )
                {

                    // 转换时间
                    _ = DateTime.TryParse(lastPush?[0]!.ToString(), out DateTime newTs);
                    endTime = DateTime.Now;//newTs;
                    startTime = newTs;//DateTime.Now;

                    int count = await _serviceTDengine.GetCount("stb_hm_bloodpress", $"ts>='{startTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and ts <='{endTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and serialno='{imei}' ");
                    // B.1.1 设备已经被重新设置, 从重置的ts 到 now 没有测量数据,inc=0
                    if (count.Equals(0))
                    {
                        _logger.LogWarning("B.1.1 设备已经被重新设置, 从重置的ts 到 now 没有测量数据,inc=0");
                        return await IotSetBloodPressCalibrationConfigResponseAsync(model.Imei, systolicRefValue, diastolicRefValue, 0, 0).ConfigureAwait(false);
                    }
                    // B.1.2 设备已经被重新设置, 从重置的ts 到 now 有测量数据
                    else
                    {
                        // 平均值: 计算去除最大值 MAX和最小值 MIN 和异常值 ABN 的平均值
                        systolicAvg = await _serviceTDengine.GetAvgExceptMaxMinValueAsync("systolic_value", "stb_hm_bloodpress", $"ts>='{startTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and ts <='{endTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and serialno='{imei}' and systolic_value < {systolicRefValue} ");
                        diastolicAvg = await _serviceTDengine.GetAvgExceptMaxMinValueAsync("diastolic_value", "stb_hm_bloodpress", $"ts>='{startTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and ts <='{endTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and serialno='{imei}' and diastolic_value < {diastolicRefValue}");
                            
                        // 数据异常,不修改增量值
                        if (systolicAvg.Equals(0) || diastolicAvg.Equals(0))
                        {
                            _logger.LogWarning("B.1.2 设备已经被重新设置, 从重置的ts 到 now 有测量数据,平均值为0,增量值将按0下发");
                            return await IotSetBloodPressCalibrationConfigResponseAsync(model.Imei, systolicRefValue, diastolicRefValue, 0, 0).ConfigureAwait(false);

                        }
                        // 数据正常
                        else
                        {
                            // 偏移参数
                            var avgOffset = 0.25M;
                            var systolicAvgOffset = avgOffset;
                            var diastolicAvgOffset = avgOffset;
                            // 增量值=（标定值-平均值）* 0.25
                            systolicInc = systolicAvg.Equals(0M) ? 0 : (int)((systolicRefValue - systolicAvg) * systolicAvgOffset)!;
                            diastolicInc = diastolicAvg.Equals(0M) ? 0 : (int)((diastolicRefValue - diastolicAvg) * diastolicAvgOffset)!;
                            _logger.LogInformation("B.1.2 设备已经被重新设置, 从重置的ts 到 now 有测量数据 ,增量值将正常下发");
                            return await IotSetBloodPressCalibrationConfigResponseAsync(imei, systolicRefValue, diastolicRefValue, systolicInc, diastolicInc).ConfigureAwait(false);
                        }

                    }

                }
                // B.2. 曾经下推过断言有测量记录
                else
                {
                    // 转换时间
                    _ = DateTime.TryParse(lastPush?[0]!.ToString(), out DateTime newTs);
                    endTime = DateTime.Now;
                    startTime = newTs;

                    // 平均值: 计算去除最大值 MAX和最小值 MIN 和异常值 ABN 的平均值
                    systolicAvg = await _serviceTDengine.GetAvgExceptMaxMinValueAsync("systolic_value", "stb_hm_bloodpress", $"ts>='{startTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and ts <='{endTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and serialno='{imei}' and systolic_value < {systolicRefValue} ");
                    diastolicAvg = await _serviceTDengine.GetAvgExceptMaxMinValueAsync("diastolic_value", "stb_hm_bloodpress", $"ts>='{startTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and ts <='{endTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and serialno='{imei}' and diastolic_value < {diastolicRefValue}");

                    // B.2.1 数据异常,将按最后一次测量值的offeset下发
                    if (systolicAvg.Equals(0) || diastolicAvg.Equals(0))
                    {
                        _logger.LogInformation("B.2.1 曾经下推过断言有测量记录,数据异常,平均值为0,增量值将按0下发");
                        return await IotSetBloodPressCalibrationConfigResponseAsync(model.Imei, systolicRefValue, diastolicRefValue, 0, 0).ConfigureAwait(false);

                    }
                    // B.2.2 数据正常,按正常计算
                    else
                    {
                        // 偏移参数
                        var avgOffset = 0.25M;
                        var systolicAvgOffset = avgOffset;
                        var diastolicAvgOffset = avgOffset;


                        // 增量值=（标定值-平均值）* 0.25
                        systolicInc = systolicAvg.Equals(0M) ? 0 : (int)((systolicRefValue - systolicAvg) * systolicAvgOffset)!;
                        diastolicInc = diastolicAvg.Equals(0M) ? 0 : (int)((diastolicRefValue - diastolicAvg) * diastolicAvgOffset)!;
                        _logger.LogInformation("B.2.2 曾经下推过断言有测量记录,数据正常,按正常计算 ,增量值将正常下发");
                        return await IotSetBloodPressCalibrationConfigResponseAsync(imei, systolicRefValue, diastolicRefValue, systolicInc, diastolicInc).ConfigureAwait(false);
                    }
                }
            }




            /**if (lastPush?.Count == 0)
            {
                #region 初始化remarks
                var resFlag = await _serviceIotWebApi.UpdatePersonRemarksAsync(imei, systolicRefValue, diastolicRefValue).ConfigureAwait(false);
                if (resFlag)
                {
                    _logger.LogInformation($"更新gps_person remarks和缓存");
                }
                #endregion
            }
            else
            {
                if (DateTime.TryParse(lastPush?[0]!.ToString(), out DateTime newTs))
                {
                    startTime = newTs;
                }
            }


            var lastBP = await _serviceTDengine.GetLastAsync("stb_hm_bloodpress", $"serialno='{imei}' order by ts desc");

            //var total = await _serviceTDengine.GetCount("stb_hm_bloodpress", $"serialno='{imei}' order by ts desc");
            //if (total >= 4)
            if (lastBP?.Count != 0)
            {
                //var systolicAggregate = await _serviceTDengine.GetAggregateValueAsync("systolic_value", "stb_hm_bloodpress", $"ts>='{startTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and ts <='{endTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and serialno='{imei}'");
                //var diastolicAggregate = await _serviceTDengine.GetAggregateValueAsync("diastolic_value", "stb_hm_bloodpress", $"ts>='{startTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and ts <='{endTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and serialno='{imei}'");
                //var systolicAggregate =  _serviceTDengine.GetAggregateValue("systolic_value", "hm_bloodpress", $"ts>='{startTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and ts <='{endTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and serialno='{bp.Serialno}'");
                //var diastolicAggregate =  _serviceTDengine.GetAggregateValue("diastolic_value", "hm_bloodpress", $"ts>='{startTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and ts <='{endTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and serialno='{bp.Serialno}'");

                //// 最大值
                //var systolicMax = systolicAggregate.Max;
                //var diastolicMax = diastolicAggregate.Max;
                //// 最小值
                //var systolicMin = systolicAggregate.Min;
                //var diastolicMin = diastolicAggregate.Min;


                // 平均值: 计算去除最大值 MAX和最小值 MIN 和异常值 ABN 的平均值
                var systolicAvg = await _serviceTDengine.GetAvgExceptMaxMinValueAsync("systolic_value", "stb_hm_bloodpress", $"ts>='{startTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and ts <='{endTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and serialno='{imei}' and systolic_value < {systolicRefValue} ");
                var diastolicAvg = await _serviceTDengine.GetAvgExceptMaxMinValueAsync("diastolic_value", "stb_hm_bloodpress", $"ts>='{startTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and ts <='{endTime:yyyy-MM-ddTHH:mm:ss.fffZ}' and serialno='{imei}' and diastolic_value < {diastolicRefValue}");

                if (systolicAvg.Equals(0) || diastolicAvg.Equals(0))
                {
                    _logger.LogWarning("平均值为0,下发增量值按0处理");
                    //return ApiResponse<object>.Fail(501, "平均值为0不处理");
                    return await IotSetBloodPressCalibrationConfigResponseAsync(imei, systolicRefValue, diastolicRefValue, 0, 0).ConfigureAwait(false);
                }
                // 偏移参数
                var avgOffset = 0.25M;
                var systolicAvgOffset = avgOffset;
                var diastolicAvgOffset = avgOffset;


                // 增量值=（标定值-平均值）* 0.25
                systolicInc = systolicAvg.Equals(0M) ? 0 : (int)((systolicRefValue - systolicAvg) * systolicAvgOffset)!;
                diastolicInc = diastolicAvg.Equals(0M) ? 0 : (int)((diastolicRefValue - diastolicAvg) * diastolicAvgOffset)!;

            }

            return await IotSetBloodPressCalibrationConfigResponseAsync(imei, systolicRefValue, diastolicRefValue, systolicInc, diastolicInc).ConfigureAwait(false);
            */
            #endregion
        }

        private async Task<ApiResponse<object>> IotSetBloodPressCalibrationConfigResponseAsync(string imei, int systolicRefValue, int diastolicRefValue, int systolicInc, int diastolicInc)
        {
            BloodPressCalibrationConfigModel bpIncData = new()
            {

                Imei = imei,
                SystolicRefValue = systolicRefValue,  //收缩压标定值，值为0 表示不生效
                DiastolicRefValue = diastolicRefValue, //舒张压标定值，值为0表示不生效
                SystolicIncValue = systolicInc, //收缩压显示增量，值为0 表示不生效
                DiastolicIncValue = diastolicInc   //舒张压显示增量，值为0 表示不生效
            };
            // 下发 IOT 增量值
            var flagIot = await _serviceIotWebApi.SetBloodPressCalibrationConfigAsync(bpIncData).ConfigureAwait(false);
            if (flagIot)
            {
                #region 保存下推记录 stb_hm_bp_push_ref_inc_value
                var sql = $"INSERT INTO health_monitor.hm_bp_push_ref_inc_value_{imei.Substring(imei.Length - 2)} " +
                        $"USING health_monitor.stb_hm_bp_push_ref_inc_value " +
                        $"TAGS ('{imei.Substring(imei.Length - 2)}')  " +
                        $"VALUES(" +
                            $"'{DateTime.Now:yyyy-MM-dd HH:mm:ss.fff}'," +
                            $"'{imei}'," +
                            $"{systolicRefValue}," +
                            $"{diastolicRefValue}," +
                            $"{systolicInc}," +
                            $"{diastolicInc}," +
                            $"{true})";

                _serviceTDengine.ExecuteInsertSQL(sql);
                #endregion

                // 返回结果
                return ApiResponse<object>.Success(bpIncData);
            }
            else
            {
                return ApiResponse<object>.Fail(500, "业务出错！下发指令失败");
            }
        }
    }
}
